/* license: AGPLv3 or later */
/* copyright 2023 zamfofex */

#include <mimimi.h>

void mimimi_linear_propagate(struct mimimi_momentum *momentum, int value, int x0, int scale, int weight)
{
	momentum->dx += ((value - x0) * 256 / scale - momentum->x) * weight / 256;
}

void mimimi_linear_apply(struct mimimi_momentum *momentum, int *value, int x0, int scale)
{
	*value = momentum->x * scale / 256 + x0;
}

void mimimi_angular_propagate(struct mimimi_momentum *momentum, struct mimimi_position *position, int x0, int y0, int a0, int scale, int weight)
{
	int a, x, y, dx;
	
	x = position->x - x0;
	y = position->y - y0;
	a = mimimi_angle(x, y);
	
	dx = (a / 64 - a0) - momentum->x * scale / 256;
	
	dx = mimimi_mod(dx, 256);
	if (dx > 128) dx = dx - 256;
	momentum->dx += dx * weight / scale;
}

void mimimi_angular_apply(struct mimimi_momentum *momentum, struct mimimi_position *position, int x0, int y0, int a0, int scale, int length)
{
	int a, x, y;
	
	a = mimimi_mod(momentum->x * scale / 256 + a0, 256);
	
	x = mimimi_sine[a] * length / 128;
	y = mimimi_cosine[a] * length / 128;
	
	position->x = x + x0;
	position->y = y + y0;
}
